package com.xyzc.start.controller;

import com.xyzc.common.core.utils.Result;
import com.xyzc.start.domain.query.AppQuery;
import com.xyzc.start.domain.vo.*;
import com.xyzc.start.service.IAppPageService;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 应用
 *
 * @author 杨兴卓
 */
@RestController
@RequestMapping("/appPage")
@RequiredArgsConstructor
@CrossOrigin
public class AppPageController {

    private final IAppPageService appPageService;

    /**
     * 行业列表
     *
     * @return
     */
    @GetMapping("/byIndustry")
    public Result<List<IndustryVo>> byIndustry() {
        List<IndustryVo> industryList = appPageService.queryByIndustry();

        return Result.ok(industryList);
    }

    /**
     * 具体行业分类的app列表
     *
     * @param careerId 行业id
     * @param appId    最后一个appId
     */
    @GetMapping("/byIndustry/{careerId}")
    public Result<AppPageCareerVo> byIndustry(@PathVariable("careerId") Long careerId, Long appId) {
        AppPageCareerVo appList = appPageService.queryByIndustryId(careerId, appId);

        return Result.ok(appList);
    }

    /**
     * 类型列表
     */
    @GetMapping("/byType")
    public Result<List<WebAppGroupVo>> byType() {
        List<WebAppGroupVo> appList = appPageService.queryByType();

        return Result.ok(appList);
    }

    /**
     * 按具体类型分类的app列表
     */
    @GetMapping("/byType/{typeId}")
    public Result<WebAppGroupVo> byType(@PathVariable("typeId") Long typeId, @Valid AppQuery appQuery) {
        WebAppGroupVo webAppGroup = appPageService.queryByTypeId(typeId, appQuery);

        return Result.ok(webAppGroup);
    }

    /**
     * 应用模糊查询
     */
    @GetMapping("/search")
    public Result<List<BaseUserSelectAppVo>> search(@Valid AppQuery appQuery) {
        List<BaseUserSelectAppVo> webAppGroupList = appPageService.queryAppByQuery(appQuery);

        return Result.ok(webAppGroupList);
    }

    /**
     * 根据id返回聊天类app
     *
     * @return Result<Void>
     */
    @GetMapping("/chat/{id}")
    public Result<Void> queryChatAppById(@PathVariable("id") Long id) {

        return Result.ok();
    }


    /**
     * 根据id返回AI文生图app
     */
    @GetMapping("/txtToPhoto/{id}")
    public Result<Void> queryAiGroupPhotoById(@PathVariable("id") Long id) {

        return Result.ok();
    }

    /**
     * 根据id返回AI图生图app
     */
    @GetMapping("/photoToPhoto/{id}")
    public Result<Void> queryAiPhotoGroupPhotoById(@PathVariable("id") Long id) {

        return Result.ok();
    }

    /**
     * 根据id返回AI数字人app
     */
    @GetMapping("/digitalHuman/{id}")
    public Result<Void> queryAiDigitalHumanById(@PathVariable("id") Long id) {

        return Result.ok();
    }

}
