package com.xyzc.start.controller;

import com.xyzc.common.core.utils.Result;
import com.xyzc.start.domain.bo.DrawModelFeedbackBo;
import com.xyzc.start.domain.query.AppQuery;
import com.xyzc.start.domain.query.DiscoveryAppQuery;
import com.xyzc.start.domain.vo.*;
import com.xyzc.start.service.IDiscoverService;
import com.xyzc.start.service.IDrawAssistantService;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.apache.http.HttpRequest;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * 发现
 *
 * @author 杨兴卓
 */
@RequiredArgsConstructor
@RestController
@RequestMapping("/discover")
@CrossOrigin
public class DiscoverPageController {

    private final IDiscoverService discoverService;

    private final IDrawAssistantService drawAssistantService;

    /**
     * 绘画广场
     */
    @GetMapping("/drawSquare")
    public Result<List<DrawGraphicsSquareVo>> drawSquare(Long graphicsId, String keyword, Long length) {
        List<DrawGraphicsSquareVo> drawGraphicsList = discoverService.queryGraphics(graphicsId, keyword, length);

        return Result.ok(drawGraphicsList);
    }


    /**
     * 绘画模型
     */
    @GetMapping("/drawModel")
    public Result<List<DrawModelCollectVo>> drawModel(Long drawModelId, String keyword, Long length) {
        List<DrawModelCollectVo> drawModelList = discoverService.queryDrawModelList(drawModelId, keyword, length);

        return Result.ok(drawModelList);
    }



    /**
     * 查询代理商邮箱
     */
    @GetMapping("/merMail")
    public Result<String> agentMail(HttpRequest request) {
        String copyRight = discoverService.queryMail(request);

        return Result.ok(copyRight);
    }

    /**
     * 分页查询大模型备案信息
     */
    @GetMapping("/filings")
    public Result<List<AiModelVo>> drawModelList(@Valid AppQuery appQuery) {
        List<AiModelVo> aiModelVoList = discoverService.queryFilingsInfo(appQuery);

        return Result.ok(aiModelVoList);
    }

    /**
     * 分页查询反向提示词
     */
    @GetMapping("/reversePrompt")
    public Result<List<DrawReversePromptVo>> reversePrompt(Long reversePromptId, String keyword, Integer length) {
        List<DrawReversePromptVo> reversePromptList = discoverService.queryReversePromptList(reversePromptId, keyword, length);

        return Result.ok(reversePromptList);
    }

    /**
     * id查询绘画详细信息
     */
    @GetMapping("/drawGraphics/{drawGraphicsId}")
    public Result<DrawGraphicsSquareVo> drawGraphicsVoDetail(@PathVariable Long drawGraphicsId) {
        DrawGraphicsSquareVo drawModelDetailVo = discoverService.queryDrawGraphicsVoDetail(drawGraphicsId);

        return Result.ok(drawModelDetailVo);
    }

    /**
     * 查询所有的绘画助手
     */
    @GetMapping("/drawAssistantList")
    public Result<List<DrawAssistantVo>> drawAssistantList() {
        List<DrawAssistantVo> drawAssistantList = drawAssistantService.queryDrawAssistantList();

        return Result.ok(drawAssistantList);
    }

    /**
     * 查询具体绘画助手的提示词
     */
    @GetMapping("/drawAssistant/{assistantId}")
    public Result<List<DrawPromptVo>> drawAssistant(@PathVariable Integer assistantId, Long promptId, String keyword, Integer length) {
        List<DrawPromptVo> drawPromptConfigList = drawAssistantService.queryDrawPromptConfigList(assistantId, promptId, keyword, length);

        return Result.ok(drawPromptConfigList);
    }


    /**
     * 应用精选、特色、玩转
     */
    @GetMapping("/selectedApp")
    public Result<List<BaseUserSelectAppVo>> selectedApp(@Valid DiscoveryAppQuery query) {
        List<BaseUserSelectAppVo> appPageList = discoverService.queryAppPageList(query);

        return Result.ok(appPageList);
    }


    /**
     * 数字人列表
     */
    @GetMapping("/digitalHuman")
    public Result<List<Void>> digitalHuman() {

        return Result.ok();
    }

    /**
     * 绘画模型反馈
     */
    @PostMapping("/drawModelFeedback")
    public Result<String> drawModelFeedback(@RequestBody DrawModelFeedbackBo bo) {
        discoverService.drawModelFeedback(bo);

        return Result.ok("反馈成功");
    }

}
