package com.xyzc.start.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xyzc.common.core.exception.ErrorCode;
import com.xyzc.common.core.exception.ServerException;
import com.xyzc.common.sms.service.ISmsService;
import com.xyzc.common.web.security.user.SecurityUser;
import com.xyzc.start.domain.*;
import com.xyzc.start.domain.vo.*;
import com.xyzc.start.mapper.*;
import com.xyzc.start.service.IComfyUIService;
import com.xyzc.start.service.IUserInfoService;
import com.xyzc.start.utils.EncryptUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.core.convert.ConversionService;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.io.File;
import java.text.ParseException;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Random;


@Service
@RequiredArgsConstructor
@Slf4j
public class ComfyUIServiceImpl implements IComfyUIService {

    public String callbackGenPicUrl="http://47.121.123.230:18660/web/comfyui/updatePromtId";

    public String comfyuiUrl = "http://10.168.1.190:43210/prompt";

    private final ComfyUIMapper baseMapper;
    // 图片系统配置类
    private final SysOssMapper sysOssMapper;
    // 短信通知
    private final ISmsService smsService;

    private final BaseUserPhotoFolderMapper photoFolderMapper;

    private final IUserInfoService userInfoService;


    /**
     * 判断推送comfyui的条件来决定是否推送
     * @param jsonObject
     * @throws ParseException
     */
    public void assert2ComfyUI2save(BaseUserPhotoFolder baseUserPhotoFolder, JSONObject jsonObject, ComfyuiExecute comfyuiExecute) {
        if(null == comfyuiExecute)
            comfyuiExecute = baseMapper.queryComfyUIExecute();
//        String executeTime = comfyUIExecutJSON.getString("executeTime");
        Date executeTime = comfyuiExecute.getExecuteTime();


        // 如果一个小时仍然没有更新，可能是已经挂了，则短信通知管理员
        // 暂时注释
        /*if (null != executeTime && comfyuiExecute.getExecuteNum() > 0 && executeTime.getTime() + 1000 * 60 * 60 < new Date().getTime()) {
            String noticePhone = comfyuiExecute.getNoticePhone();
            // 发送短信
            smsService.sendCode(noticePhone, "code", "comfyui生图生成异常！");
        }*/
        String promptId = "";
        // 如果为0，则进行推送，并且更新执行次数，
//        if (null == comfyuiExecute || 0 == comfyuiExecute.getExecuteNum()) {
            // 更新执行次数
            promptId = sendRestTemplate(jsonObject);
//        }

        Long ossId = null;
        if(Strings.isNotBlank(promptId)) {
//            ComfyuiExecute comfyuiExecute = new ComfyuiExecute();
            // 暂时注释
//            comfyuiExecute.setPromptId(promptId);
//            comfyuiExecute.setExecuteNum(comfyuiExecute.getExecuteNum() + 1);
//            comfyuiExecute.setExecuteTime(new Date());
//            int i = baseMapper.updateComfyuiExecute(comfyuiExecute);

            JSONObject minioJson = sysOssMapper.queryConfig("1", "minio");
            String bucketName = minioJson.getString("bucket_name");
            // 保存oss表
            SysOss sysOss = new SysOss();
            sysOss.setPromptId(promptId);
            sysOss.setService(bucketName);
            sysOss.setUrl("");
            sysOss.setFileName("");
            sysOss.setOriginalName("");
            sysOss.setFileSuffix("");
            sysOssMapper.insert(sysOss);
            ossId = sysOss.getOssId();
        }
        // 保存用户工作流日志
//            saveUserWorkflow(comfyuiUserWorkflow, jsonObject, promptId);
        baseUserPhotoFolder.setWorkflowJson(jsonObject.toJSONString());
        baseUserPhotoFolder.setPromptId(promptId);
        baseUserPhotoFolder.setIsGenerate(0);
        baseUserPhotoFolder.setCreateTime(new Date());
//        photoFolderMapper.insertOrUpdate(baseUserPhotoFolder);
        // 图片处理
        if(Arrays.asList(1,2,3,4,5,6,7,8,9).contains(baseUserPhotoFolder.getPicType()))
            baseUserPhotoFolder.setType(1l);
        else // SD类型
            baseUserPhotoFolder.setType(2l);
        baseUserPhotoFolder.setPhotoOssId(ossId);
        baseUserPhotoFolder.setBaseUserId(SecurityUser.getUserId());
        photoFolderMapper.insertOrUpdate(baseUserPhotoFolder);
    }

    /**
     * 对comfyui生图的回调请求，进行保存图片的url
     * 再次推送userworkflow的，并记录到执行表中
     * @param userId
     * @param minioUrl
     * @param promptId
     */
    @Override
    public void updatePromtId(Long userId, String minioUrl, String promptId) {
        JSONObject jsonObject = sysOssMapper.queryConfig("1", "minio");
        if (jsonObject == null || jsonObject.getString("bucket_name") == null) {
            log.error("请先配置minio信息");
            throw new ServerException(ErrorCode.COMFYUI_MINIO_CONFIG);
        }

        String url = "http://" + jsonObject.getString("endpoint") + File.separator + jsonObject.getString("bucket_name") + File.separator + minioUrl;
///comfyui/generate/a46073be387e462eb819e1096cf65dc9.png
        SysOss sysOss = sysOssMapper.selectOne(new QueryWrapper<SysOss>().eq("prompt_id", promptId));
        if (sysOss == null)
            sysOss = new SysOss();
        if (minioUrl.lastIndexOf(".") >= 0) {
            int lastIndexOf = minioUrl.lastIndexOf(".");
            sysOss.setFileSuffix(minioUrl.substring(lastIndexOf));
        }
        sysOss.setFileName(minioUrl);
        sysOss.setUrl(url);
        sysOss.setService(jsonObject.getString("bucket_name"));
        sysOss.setPromptId(promptId);
        sysOssMapper.insertOrUpdate(sysOss);

        ComfyuiExecute comfyuiExecute = baseMapper.queryComfyUIExecute();
        comfyuiExecute.setExecuteNum(comfyuiExecute.getExecuteNum() - 1);
        comfyuiExecute.setExecuteTime(null);
        comfyuiExecute.setPromptId(null);
        baseMapper.updateComfyuiExecute(comfyuiExecute);

        // 扣除燃料
//        userInfoService.deduct(null, 1808705496965525505l);

        // 获取表，进行判断再推送
//        BaseUserPhotoFolder baseUserPhotoFolder = photoFolderMapper.selectOne(new QueryWrapper<BaseUserPhotoFolder>().eq("base_user_id", userId).eq("prompt_id", promptId)
//                .select("is_generate","generate_time"));
        BaseUserPhotoFolder baseUserPhotoFolder = photoFolderMapper.queryPhotoFolderByPromptId(userId, promptId);
//                .select("is_generate","generate_time"));
        baseUserPhotoFolder.setGenerateTime(new Date());
        baseUserPhotoFolder.setIsGenerate(3);
        photoFolderMapper.updateById(baseUserPhotoFolder);

//        photoFolderMapper.selectOne(new QueryWrapper<>().eq("", comfyuiUserWorkflow.getId()))

        // 获取排队生成图的workflow, 进行推送comfyui
//        BaseUserPhotoFolder baseUserPhotoFolders = photoFolderMapper.selectOne(new QueryWrapper<BaseUserPhotoFolder>().eq("is_generate", 0)
//                .select("workflow_json", "prompt_id", "is_generate", "generate_time", "type", "photo_oss_id").orderByAsc("create_time").last("limit 1"));
        BaseUserPhotoFolder baseUserPhotoFolders = photoFolderMapper.queryPhotoFolderByIsGenerate();
//        baseUserPhotoFolder.setWorkflowJson(jsonObject.toJSONString());
//        baseUserPhotoFolder.setPromptId(promptId);
//        baseUserPhotoFolder.setIsGenerate(0);
//        baseUserPhotoFolder.setCreateTime(new Date());
////        photoFolderMapper.insertOrUpdate(baseUserPhotoFolder);
//        // 图片处理
//        if(Arrays.asList(1,2,3,4,5,6,7,8,9).contains(baseUserPhotoFolder.getPicType()))
//            baseUserPhotoFolder.setType(1l);
//        else // SD类型
//            baseUserPhotoFolder.setType(2l);
//        baseUserPhotoFolder.setPhotoOssId(ossId);

//        if(null != baseUserPhotoFolders){
//            // 如果不为空，那么直接再进行推送
//            assert2ComfyUI2save(baseUserPhotoFolders, JSONObject.parseObject(baseUserPhotoFolders.getWorkflowJson()), comfyuiExecute);
//        }
    }

    /**
     * 定时器执行， 防止某些因异常导致没有执行成功，定时器重新执行
     */
    @Override
    public void quartz2ComfyUI(){
        ComfyuiExecute comfyuiExecute = baseMapper.queryComfyUIExecute();
        if (null != comfyuiExecute || 0 == comfyuiExecute.getExecuteNum()) {
            // 获取排队生成图的workflow, 进行推送comfyui  // 再查询 有promptId情况下，如果超过了1个小时 没以后生成的，那么直接执行
            BaseUserPhotoFolder baseUserPhotoFolder = photoFolderMapper.selectOne(new QueryWrapper<BaseUserPhotoFolder>()
                    .eq("is_generate", 0).eq("del_flag", "0").orderByAsc("create_time").last("limit 1"));
            if (null != baseUserPhotoFolder) {
                // 如果不为空，那么直接再进行推送
                assert2ComfyUI2save(baseUserPhotoFolder, JSONObject.parseObject(baseUserPhotoFolder.getWorkflowJson()), comfyuiExecute);
            }

        }
    }

    @Override
    public List<ComfyuiWorkflowVo> queryAllWorkflow() {
        log.info("查询comfyUI图片模板");
        List<ComfyuiWorkflowVo> comfyuiWorkflows = baseMapper.queryAllWorkflow();
        return comfyuiWorkflows;
    }

    @Override
    public List<BaseUserPhotoFolder> comfyUIGeneratPic(BaseUserPhotoFolder baseUserPhotoFolder) throws ParseException {

        Integer picType = baseUserPhotoFolder.getPicType();
        if(null == picType){
            throw new ServerException(ErrorCode.COMFYUI_NOT_MODEL_TYPE);
        }
        if (!Arrays.asList(1, 2, /*3,*/ 4, 5, 6, 7, 8, 9).contains(picType)) {
            throw new ServerException(ErrorCode.COMFYUI_NOT_MODEL_TYPE);
        }
        String picUrl = baseUserPhotoFolder.getUploadPicUrl();
        if(Strings.isBlank(picUrl)){
            throw new ServerException(ErrorCode.COMFYUI_UP_PIC);
        }

        ComfyuiWorkflow comfyuiWorkflow = baseMapper.queryWorkflowById(picType);
        baseUserPhotoFolder.setWorkflowId(comfyuiWorkflow.getId());
        baseUserPhotoFolder.setBaseUserId(SecurityUser.getUserId());
        if (Arrays.asList(1, 2, 4, 6, 7).contains(picType)) {
            log.info("发送工作流1、2、4、6、7、9");
            sendWorkFlow(picType, picUrl, baseUserPhotoFolder, comfyuiWorkflow);
        } else if (3 == picType) {
            log.info("发送工作流3");
//            sendWorkFlow3(picType, picUrl, comfyuiWorkflow, jsonObject);//二维码制作
        } else if (5 == picType) {
            log.info("发送工作流5");
            sendWorkFlow5(baseUserPhotoFolder, comfyuiWorkflow);//证件照制作
        } else if (8 == picType) {
            log.info("发送工作流8");
            sendWorkFlow8(baseUserPhotoFolder, comfyuiWorkflow);//尺寸外扩
        } else if (9 == picType) {
            sendWorkFlow9(baseUserPhotoFolder, comfyuiWorkflow);
        }
        return null;
    }

    /**
     * 插入用户工作流表
     * @param comfyuiUserWorkflowBo
     */
    public void insertUserWorkFlow(ComfyuiUserWorkflow comfyuiUserWorkflowBo){
        Long userId = SecurityUser.getUserId();
        ComfyuiUserWorkflow comfyuiUserWorkflow = new ComfyuiUserWorkflow();
        comfyuiUserWorkflow.setUserId(userId);
        comfyuiUserWorkflow.setWorkflowJson(comfyuiUserWorkflowBo.getWorkflowJson());
        comfyuiUserWorkflow.setGenerateTime(new Date());

        baseMapper.insert(comfyuiUserWorkflow);
    }

    /**
     * 推送SD
     * @param baseUserPhotoFolder
     * @return
     */
    @Override
    public List<BaseUserPhotoFolder> sdGeneratePic(BaseUserPhotoFolder baseUserPhotoFolder) {
        Integer picType = baseUserPhotoFolder.getPicType();
        if(null == picType){
            throw new ServerException(ErrorCode.COMFYUI_NOT_MODEL_TYPE);
        }
        if (!Arrays.asList(10000, 10001).contains(picType)) {
            throw new ServerException(ErrorCode.COMFYUI_NOT_MODEL_TYPE);
        }
        String picUrl = baseUserPhotoFolder.getUploadPicUrl();
        if(Strings.isBlank(picUrl) && picType == 10000){
            throw new ServerException(ErrorCode.COMFYUI_UP_PIC);
        }
        if (Strings.isBlank(baseUserPhotoFolder.getPromptJson())) {
            throw new ServerException(ErrorCode.COMFYUI_INPUT_SIZES);
        }

        JSONObject jsonObject = JSONObject.parseObject(baseUserPhotoFolder.getPromptJson());

        ComfyuiWorkflow comfyuiWorkflow = baseMapper.queryWorkflowById(picType);
        String workflowJson = comfyuiWorkflow.getWorkflowJson();
        String appKey = comfyuiWorkflow.getAppKey();


        if (10000 == picType) {
            workdFlowSd(baseUserPhotoFolder, comfyuiWorkflow);

        } else {
            wordSD_NoPic(baseUserPhotoFolder, comfyuiWorkflow);
        }

        return null;
    }

    @Override
    public List<ComfyuiModel> getAllModel(ComfyuiModel comfyuiModel) {
        return List.of();
    }

    /**
     * 1	商品抠图
     * 2	图片清晰
     * 3	二维码制作-先不做
     * 4	老照片上色
     * 5	证件照制作
     * 6	卡通自拍
     * 7	头像抠图
     * 8	尺寸外扩
     * 9	背景替换
     * 10000	SD制作
     * 10001	SD制作-无图片
     * @param picType
     * @param picUrl
     * @param comfyuiWorkflow
     */
    public void sendWorkFlow(int picType, String picUrl,BaseUserPhotoFolder baseUserPhotoFolder, ComfyuiWorkflow comfyuiWorkflow) throws ParseException {
        String appKey = comfyuiWorkflow.getAppKey();
        String workFlowStr = comfyuiWorkflow.getWorkflowJson();
        JSONObject jsonObject = getWorkFlowJSON(workFlowStr, appKey, picType);
        jsonObject.put("minio_url", picUrl);
        jsonObject.put("user_workflow_id", picType);
        //
        assert2ComfyUI2save(baseUserPhotoFolder, jsonObject, null);
    }

    // 二维码制作-先不做
    public void sendWorkFlow3(int picType, String picUrl, ComfyuiWorkflow comfyuiWorkflowVo, JSONObject paramObject) throws ParseException {
        String workFlowStr = "{\"client_id\": \"用户ID\",\"user_workflow_id\": \"用户生成图片的记录id\",\"pic_type\": \"图片处理类型\",\"oss_url\": \"写入oss图片要生成的地址\",\"random\": \"填入随机数\",\"gen_key\": \"生成key\",\"callback_gen_pic_url\": \"输入生成图片上传oss的地址\", \"prompt\": {\"23\": {\"inputs\": {\"image\": \"下载 (1) (1).png\", \"upload\": \"image\"}, \"class_type\": \"LoadImage\"}, \"69\": {\"inputs\": {\"images\": [\"72\", 0]}, \"class_type\": \"PreviewImage\"}, \"71\": {\"inputs\": {\"model_name\": \"大sam_vit_b_01ec64.pth\", \"device_mode\": \"Prefer GPU\"}, \"class_type\": \"SAMLoader\"}, \"72\": {\"inputs\": {\"guide_size\": 256, \"guide_size_for\": \"True\", \"max_size\": 768, \"seed\": 326434926972545, \"steps\": 20, \"cfg\": 8, \"sampler_name\": \"dpmpp_3m_sde_gpu\", \"scheduler\": \"karras\", \"denoise\": 0.5, \"feather\": 5, \"noise_mask\": \"True\", \"force_inpaint\": \"False\", \"bbox_threshold\": 0.5, \"bbox_dilation\": 10, \"bbox_crop_factor\": 3, \"sam_detection_hint\": \"center-1\", \"sam_dilation\": 0, \"sam_threshold\": 0.93, \"sam_bbox_expansion\": 0, \"sam_mask_hint_threshold\": 0.7, \"sam_mask_hint_use_negative\": \"False\", \"drop_size\": 10, \"wildcard\": \"\", \"cycle\": 1, \"inpaint_model\": \"False\", \"noise_mask_feather\": 20, \"image\": [\"127\", 5], \"model\": [\"127\", 0], \"clip\": [\"128\", 5], \"vae\": [\"127\", 4], \"positive\": [\"127\", 1], \"negative\": [\"127\", 2], \"bbox_detector\": [\"79\", 0], \"sam_model_opt\": [\"71\", 0], \"segm_detector_opt\": [\"79\", 1]}, \"class_type\": \"FaceDetailer\"}, \"79\": {\"inputs\": {\"model_name\": \"segm/person_yolov8m-seg.pt\"}, \"class_type\": \"UltralyticsDetectorProvider\"}, \"105\": {\"inputs\": {\"upscale_by\": 2, \"seed\": 847631375032270, \"steps\": 15, \"cfg\": 6.3333740234375, \"sampler_name\": \"dpmpp_2m_sde_gpu\", \"scheduler\": \"karras\", \"denoise\": 0.2966662597656252, \"mode_type\": \"Linear\", \"tile_width\": 768, \"tile_height\": 768, \"mask_blur\": 8, \"tile_padding\": 32, \"seam_fix_mode\": \"None\", \"seam_fix_denoise\": 1, \"seam_fix_width\": 64, \"seam_fix_mask_blur\": 8, \"seam_fix_padding\": 16, \"force_uniform_tiles\": \"enable\", \"tiled_decode\": \"False\", \"image\": [\"72\", 0], \"model\": [\"127\", 0], \"positive\": [\"127\", 1], \"negative\": [\"127\", 2], \"vae\": [\"127\", 4], \"upscale_model\": [\"106\", 0]}, \"class_type\": \"UltimateSDUpscale\"}, \"106\": {\"inputs\": {\"model_name\": \"4x-UltraSharp.pth\"}, \"class_type\": \"UpscaleModelLoader\"}, \"113\": {\"inputs\": {\"switch_1\": \"On\", \"controlnet_1\": \"control_v1p_sd15_qrcode_monster.safetensors\", \"controlnet_strength_1\": 0.9, \"start_percent_1\": 0, \"end_percent_1\": 1, \"switch_2\": \"On\", \"controlnet_2\": \"control_v1p_sd15_brightness.safetensors\", \"controlnet_strength_2\": 0.1, \"start_percent_2\": 0, \"end_percent_2\": 1, \"switch_3\": \"On\", \"controlnet_3\": \"QRPattern_v2_9500.safetensors\", \"controlnet_strength_3\": 0.5, \"start_percent_3\": 0, \"end_percent_3\": 1, \"image_1\": [\"131\", 0], \"image_2\": [\"131\", 0], \"image_3\": [\"131\", 0]}, \"class_type\": \"CR Multi-ControlNet Stack\"}, \"120\": {\"inputs\": {\"filename_prefix\": \"ComfyUI\", \"images\": [\"105\", 0]}, \"class_type\": \"SaveImage\"}, \"127\": {\"inputs\": {\"seed\": 487314088424544, \"steps\": 30, \"cfg\": 7, \"sampler_name\": \"dpmpp_2m_sde_gpu\", \"scheduler\": \"karras\", \"denoise\": 1, \"preview_method\": \"none\", \"vae_decode\": \"true\", \"model\": [\"128\", 0], \"positive\": [\"128\", 1], \"negative\": [\"128\", 2], \"latent_image\": [\"128\", 3], \"optional_vae\": [\"128\", 4]}, \"class_type\": \"KSampler (Efficient)\"}, \"128\": {\"inputs\": {\"ckpt_name\": \"文-图\\\\二次元\\\\二次元10.safetensors\", \"vae_name\": \"vae-ft-mse-840000-ema-pruned.safetensors\", \"clip_skip\": -1, \"lora_name\": \"None\", \"lora_model_strength\": 1, \"lora_clip_strength\": 1, \"positive\": [\"129\", 0], \"negative\": [\"130\", 0], \"token_normalization\": \"none\", \"weight_interpretation\": \"comfy\", \"empty_latent_width\": 768, \"empty_latent_height\": 768, \"batch_size\": 1, \"cnet_stack\": [\"113\", 0]}, \"class_type\": \"Efficient Loader\"}, \"129\": {\"inputs\": {\"prompt\": \"Scif vibes. Otherworldly. Cinematic. Ominous mountain, digital art, inspired by Cyril Rolando, digital art, blood red moon, forest, Japanese temple, beeple and jeremiah ketner, symmetrical digital illustration, realism | beeple, over detailed art, music album art. Creepy\\n\\n\"}, \"class_type\": \"CR Prompt Text\"}, \"130\": {\"inputs\": {\"prompt\": \"nsfw,embedding:quality\\\\EasyNegativeV2, embedding:quality\\\\FastNegativeV2, \\njpeg artifacts,signature,watermark,username,\"}, \"class_type\": \"CR Prompt Text\"}, \"131\": {\"inputs\": {\"Input\": 2, \"image2\": [\"23\", 0]}, \"class_type\": \"CR Image Input Switch\"}, \"132\": {\"inputs\": {\"images\": [\"131\", 0]}, \"class_type\": \"PreviewImage\"}}, \"extra_data\": {\"extra_pnginfo\": {\"workflow\": {\"last_node_id\": 152, \"last_link_id\": 492, \"nodes\": [{\"id\": 72, \"type\": \"FaceDetailer\", \"pos\": [1305.4420893359375, -955], \"size\": {\"0\": 456, \"1\": 880}, \"flags\": {\"pinned\": \"False\"}, \"order\": 12, \"mode\": 0, \"inputs\": [{\"name\": \"image\", \"type\": \"IMAGE\", \"link\": 440, \"label\": \"图像\"}, {\"name\": \"model\", \"type\": \"MODEL\", \"link\": 458, \"label\": \"模型\"}, {\"name\": \"clip\", \"type\": \"CLIP\", \"link\": 434, \"label\": \"CLIP\"}, {\"name\": \"vae\", \"type\": \"VAE\", \"link\": 461, \"label\": \"VAE\"}, {\"name\": \"positive\", \"type\": \"CONDITIONING\", \"link\": 459, \"label\": \"正面条件\"}, {\"name\": \"negative\", \"type\": \"CONDITIONING\", \"link\": 460, \"label\": \"负面条件\"}, {\"name\": \"bbox_detector\", \"type\": \"BBOX_DETECTOR\", \"link\": 234, \"label\": \"BBox检测\"}, {\"name\": \"sam_model_opt\", \"type\": \"SAM_MODEL\", \"link\": 153, \"label\": \"SAM模型\"}, {\"name\": \"segm_detector_opt\", \"type\": \"SEGM_DETECTOR\", \"link\": 446, \"label\": \"Segm检测\"}, {\"name\": \"detailer_hook\", \"type\": \"DETAILER_HOOK\", \"link\": \"None\", \"label\": \"细化约束\"}], \"outputs\": [{\"name\": \"image\", \"type\": \"IMAGE\", \"links\": [207, 444], \"shape\": 3, \"slot_index\": 0, \"label\": \"图像\"}, {\"name\": \"cropped_refined\", \"type\": \"IMAGE\", \"links\": \"None\", \"shape\": 6, \"label\": \"细化图像\"}, {\"name\": \"cropped_enhanced_alpha\", \"type\": \"IMAGE\", \"links\": \"None\", \"shape\": 6, \"label\": \"细化部分\"}, {\"name\": \"mask\", \"type\": \"MASK\", \"links\": \"None\", \"shape\": 3, \"label\": \"遮罩\"}, {\"name\": \"detailer_pipe\", \"type\": \"DETAILER_PIPE\", \"links\": \"None\", \"shape\": 3, \"label\": \"细化节点束\"}, {\"name\": \"cnet_images\", \"type\": \"IMAGE\", \"links\": \"None\", \"shape\": 6, \"label\": \"ControlNet图像\"}], \"properties\": {\"Node name for S&R\": \"FaceDetailer\"}, \"widgets_values\": [256, \"True\", 768, 326434926972545, \"randomize\", 20, 8, \"dpmpp_3m_sde_gpu\", \"karras\", 0.5, 5, \"True\", \"False\", 0.5, 10, 3, \"center-1\", 0, 0.93, 0, 0.7, \"False\", 10, \"\", 1, \"False\", 20], \"color\": \"#432\", \"bgcolor\": \"#653\"}, {\"id\": 106, \"type\": \"UpscaleModelLoader\", \"pos\": [755, -876], \"size\": {\"0\": 315, \"1\": 58}, \"flags\": {\"pinned\": \"False\"}, \"order\": 0, \"mode\": 0, \"outputs\": [{\"name\": \"UPSCALE_MODEL\", \"type\": \"UPSCALE_MODEL\", \"links\": [370], \"shape\": 3, \"label\": \"放大模型\"}], \"properties\": {\"Node name for S&R\": \"UpscaleModelLoader\"}, \"widgets_values\": [\"4x-UltraSharp.pth\"], \"color\": \"#432\", \"bgcolor\": \"#653\"}, {\"id\": 131, \"type\": \"CR Image Input Switch\", \"pos\": [1111.6924296417799, 24.598378716021728], \"size\": {\"0\": 291.67889404296875, \"1\": 78}, \"flags\": {}, \"order\": 7, \"mode\": 0, \"inputs\": [{\"name\": \"image1\", \"type\": \"IMAGE\", \"link\": \"None\", \"label\": \"图像1\"}, {\"name\": \"image2\", \"type\": \"IMAGE\", \"link\": 492, \"label\": \"图像2\"}], \"outputs\": [{\"name\": \"IMAGE\", \"type\": \"IMAGE\", \"links\": [448, 449, 450, 468], \"shape\": 3, \"slot_index\": 0, \"label\": \"图像\"}, {\"name\": \"show_help\", \"type\": \"STRING\", \"links\": \"None\", \"shape\": 3}], \"properties\": {\"Node name for S&R\": \"CR Image Input Switch\"}, \"widgets_values\": [2], \"color\": \"#81e6f3\", \"bgcolor\": \"#6dd2df\"}, {\"id\": 71, \"type\": \"SAMLoader\", \"pos\": [744, -626], \"size\": {\"0\": 315, \"1\": 82}, \"flags\": {\"pinned\": \"False\"}, \"order\": 1, \"mode\": 0, \"outputs\": [{\"name\": \"SAM_MODEL\", \"type\": \"SAM_MODEL\", \"links\": [153], \"shape\": 3, \"slot_index\": 0, \"label\": \"SAM模型\"}], \"properties\": {\"Node name for S&R\": \"SAMLoader\"}, \"widgets_values\": [\"大sam_vit_b_01ec64.pth\", \"Prefer GPU\"], \"color\": \"#2a363b\", \"bgcolor\": \"#3f5159\"}, {\"id\": 79, \"type\": \"UltralyticsDetectorProvider\", \"pos\": [750, -758], \"size\": {\"0\": 315, \"1\": 78}, \"flags\": {\"pinned\": \"False\"}, \"order\": 2, \"mode\": 0, \"outputs\": [{\"name\": \"BBOX_DETECTOR\", \"type\": \"BBOX_DETECTOR\", \"links\": [234], \"shape\": 3, \"slot_index\": 0, \"label\": \"BBox检测\"}, {\"name\": \"SEGM_DETECTOR\", \"type\": \"SEGM_DETECTOR\", \"links\": [446], \"shape\": 3, \"slot_index\": 1, \"label\": \"SEGM检测\"}], \"properties\": {\"Node name for S&R\": \"UltralyticsDetectorProvider\"}, \"widgets_values\": [\"segm/person_yolov8m-seg.pt\"], \"color\": \"#2a363b\", \"bgcolor\": \"#3f5159\"}, {\"id\": 105, \"type\": \"UltimateSDUpscale\", \"pos\": [1922, -938], \"size\": {\"0\": 315, \"1\": 802}, \"flags\": {\"pinned\": \"False\"}, \"order\": 14, \"mode\": 0, \"inputs\": [{\"name\": \"image\", \"type\": \"IMAGE\", \"link\": 444, \"label\": \"图像\"}, {\"name\": \"model\", \"type\": \"MODEL\", \"link\": 462, \"label\": \"模型\"}, {\"name\": \"positive\", \"type\": \"CONDITIONING\", \"link\": 463, \"label\": \"正面条件\"}, {\"name\": \"negative\", \"type\": \"CONDITIONING\", \"link\": 464, \"label\": \"负面条件\"}, {\"name\": \"vae\", \"type\": \"VAE\", \"link\": 465, \"label\": \"VAE\"}, {\"name\": \"upscale_model\", \"type\": \"UPSCALE_MODEL\", \"link\": 370, \"slot_index\": 5, \"label\": \"放大模型\"}], \"outputs\": [{\"name\": \"IMAGE\", \"type\": \"IMAGE\", \"links\": [418], \"shape\": 3, \"slot_index\": 0, \"label\": \"图像\"}], \"properties\": {\"Node name for S&R\": \"UltimateSDUpscale\"}, \"widgets_values\": [2, 847631375032270, \"randomize\", 15, 6.3333740234375, \"dpmpp_2m_sde_gpu\", \"karras\", 0.2966662597656252, \"Linear\", 768, 768, 8, 32, \"None\", 1, 64, 8, 16, \"enable\", \"False\"], \"color\": \"#233\", \"bgcolor\": \"#355\"}, {\"id\": 135, \"type\": \"PrimitiveNode\", \"pos\": [740, -89], \"size\": {\"0\": 210, \"1\": 82}, \"flags\": {}, \"order\": 3, \"mode\": 0, \"outputs\": [{\"name\": \"INT\", \"type\": \"INT\", \"links\": [469], \"widget\": {\"name\": \"seed\"}}], \"properties\": {\"Run widget replace on values\": \"False\"}, \"widgets_values\": [487314088424544, \"randomize\"]}, {\"id\": 120, \"type\": \"SaveImage\", \"pos\": [1967, -35], \"size\": {\"0\": 532.260498046875, \"1\": 573.1259155273438}, \"flags\": {}, \"order\": 15, \"mode\": 0, \"inputs\": [{\"name\": \"images\", \"type\": \"IMAGE\", \"link\": 418, \"label\": \"图像\"}], \"properties\": {\"Node name for S&R\": \"SaveImage\"}, \"widgets_values\": [\"ComfyUI\"]}, {\"id\": 69, \"type\": \"PreviewImage\", \"pos\": [180, -447], \"size\": {\"0\": 365.0760803222656, \"1\": 354.8205871582031}, \"flags\": {\"pinned\": \"False\"}, \"order\": 13, \"mode\": 0, \"inputs\": [{\"name\": \"images\", \"type\": \"IMAGE\", \"link\": 207, \"label\": \"图像\"}], \"properties\": {\"Node name for S&R\": \"PreviewImage\"}, \"color\": \"#223\", \"bgcolor\": \"#335\"}, {\"id\": 132, \"type\": \"PreviewImage\", \"pos\": [1106, 179], \"size\": {\"0\": 328.13348388671875, \"1\": 339.2913818359375}, \"flags\": {}, \"order\": 9, \"mode\": 0, \"inputs\": [{\"name\": \"images\", \"type\": \"IMAGE\", \"link\": 468, \"label\": \"图像\"}], \"properties\": {\"Node name for S&R\": \"PreviewImage\"}}, {\"id\": 127, \"type\": \"KSampler (Efficient)\", \"pos\": [740, -474], \"size\": {\"0\": 325, \"1\": 350}, \"flags\": {}, \"order\": 11, \"mode\": 0, \"inputs\": [{\"name\": \"model\", \"type\": \"MODEL\", \"link\": 452, \"label\": \"模型\"}, {\"name\": \"positive\", \"type\": \"CONDITIONING\", \"link\": 453, \"label\": \"正面条件\"}, {\"name\": \"negative\", \"type\": \"CONDITIONING\", \"link\": 454, \"label\": \"负面条件\"}, {\"name\": \"latent_image\", \"type\": \"LATENT\", \"link\": 490, \"label\": \"Latent\", \"slot_index\": 3}, {\"name\": \"optional_vae\", \"type\": \"VAE\", \"link\": 455, \"label\": \"VAE(可选)\"}, {\"name\": \"script\", \"type\": \"SCRIPT\", \"link\": \"None\", \"label\": \"脚本\"}, {\"name\": \"seed\", \"type\": \"INT\", \"link\": 469, \"widget\": {\"name\": \"seed\"}, \"label\": \"随机种\", \"slot_index\": 6}], \"outputs\": [{\"name\": \"MODEL\", \"type\": \"MODEL\", \"links\": [458, 462], \"shape\": 3, \"slot_index\": 0, \"label\": \"模型\"}, {\"name\": \"CONDITIONING+\", \"type\": \"CONDITIONING\", \"links\": [459, 463], \"shape\": 3, \"slot_index\": 1, \"label\": \"正面条件\"}, {\"name\": \"CONDITIONING-\", \"type\": \"CONDITIONING\", \"links\": [460, 464], \"shape\": 3, \"slot_index\": 2, \"label\": \"负面条件\"}, {\"name\": \"LATENT\", \"type\": \"LATENT\", \"links\": \"None\", \"shape\": 3, \"label\": \"Latent\"}, {\"name\": \"VAE\", \"type\": \"VAE\", \"links\": [461, 465], \"shape\": 3, \"slot_index\": 4, \"label\": \"VAE\"}, {\"name\": \"IMAGE\", \"type\": \"IMAGE\", \"links\": [440], \"shape\": 3, \"slot_index\": 5, \"label\": \"图像\"}], \"properties\": {\"Node name for S&R\": \"KSampler (Efficient)\"}, \"widgets_values\": [487314088424544, \"None\", 30, 7, \"dpmpp_2m_sde_gpu\", \"karras\", 1, \"none\", \"true\"], \"color\": \"#332233\", \"bgcolor\": \"#553355\", \"shape\": 1}, {\"id\": 113, \"type\": \"CR Multi-ControlNet Stack\", \"pos\": [1474, 38], \"size\": {\"0\": 418.1999816894531, \"1\": 454}, \"flags\": {\"pinned\": \"False\"}, \"order\": 8, \"mode\": 0, \"inputs\": [{\"name\": \"image_1\", \"type\": \"IMAGE\", \"link\": 449, \"label\": \"图像1\"}, {\"name\": \"image_2\", \"type\": \"IMAGE\", \"link\": 448, \"label\": \"图像2\"}, {\"name\": \"image_3\", \"type\": \"IMAGE\", \"link\": 450, \"label\": \"图像3\"}, {\"name\": \"controlnet_stack\", \"type\": \"CONTROL_NET_STACK\", \"link\": \"None\", \"slot_index\": 3, \"label\": \"ControlNet堆\"}], \"outputs\": [{\"name\": \"CONTROLNET_STACK\", \"type\": \"CONTROL_NET_STACK\", \"links\": [443], \"shape\": 3, \"slot_index\": 0, \"label\": \"ControlNet堆\"}, {\"name\": \"show_help\", \"type\": \"STRING\", \"links\": \"None\", \"shape\": 3}], \"properties\": {\"Node name for S&R\": \"CR Multi-ControlNet Stack\"}, \"widgets_values\": [\"On\", \"control_v1p_sd15_qrcode_monster.safetensors\", 0.9, 0, 1, \"On\", \"control_v1p_sd15_brightness.safetensors\", 0.1, 0, 1, \"On\", \"QRPattern_v2_9500.safetensors\", 0.5, 0, 1], \"color\": \"#432\", \"bgcolor\": \"#653\"}, {\"id\": 129, \"type\": \"CR Prompt Text\", \"pos\": [560, 254], \"size\": {\"0\": 394.2886657714844, \"1\": 120.40887451171875}, \"flags\": {}, \"order\": 4, \"mode\": 0, \"outputs\": [{\"name\": \"prompt\", \"type\": \"STRING\", \"links\": [430], \"shape\": 3, \"slot_index\": 0, \"label\": \"提示词文本\"}, {\"name\": \"show_help\", \"type\": \"STRING\", \"links\": \"None\", \"shape\": 3}], \"properties\": {\"Node name for S&R\": \"CR Prompt Text\"}, \"widgets_values\": [\"Scif vibes. Otherworldly. Cinematic. Ominous mountain, digital art, inspired by Cyril Rolando, digital art, blood red moon, forest, Japanese temple, beeple and jeremiah ketner, symmetrical digital illustration, realism | beeple, over detailed art, music album art. Creepy\\n\\n\"], \"color\": \"#232\", \"bgcolor\": \"#353\"}, {\"id\": 130, \"type\": \"CR Prompt Text\", \"pos\": [560, 434], \"size\": {\"0\": 400.0119323730469, \"1\": 96}, \"flags\": {}, \"order\": 5, \"mode\": 0, \"outputs\": [{\"name\": \"prompt\", \"type\": \"STRING\", \"links\": [431], \"shape\": 3, \"slot_index\": 0, \"label\": \"提示词文本\"}, {\"name\": \"show_help\", \"type\": \"STRING\", \"links\": [], \"shape\": 3}], \"properties\": {\"Node name for S&R\": \"CR Prompt Text\"}, \"widgets_values\": [\"nsfw,embedding:quality\\\\EasyNegativeV2, embedding:quality\\\\FastNegativeV2, \\njpeg artifacts,signature,watermark,username,\"], \"color\": \"#2a363b\", \"bgcolor\": \"#3f5159\"}, {\"id\": 128, \"type\": \"Efficient Loader\", \"pos\": [189, -962], \"size\": {\"0\": 400, \"1\": 462}, \"flags\": {}, \"order\": 10, \"mode\": 0, \"inputs\": [{\"name\": \"lora_stack\", \"type\": \"LORA_STACK\", \"link\": \"None\", \"label\": \"LoRA堆\"}, {\"name\": \"cnet_stack\", \"type\": \"CONTROL_NET_STACK\", \"link\": 443, \"label\": \"ControlNet堆\"}, {\"name\": \"positive\", \"type\": \"STRING\", \"link\": 430, \"widget\": {\"name\": \"positive\"}, \"label\": \"正面条件\"}, {\"name\": \"negative\", \"type\": \"STRING\", \"link\": 431, \"widget\": {\"name\": \"negative\"}, \"label\": \"负面条件\"}], \"outputs\": [{\"name\": \"MODEL\", \"type\": \"MODEL\", \"links\": [452], \"shape\": 3, \"slot_index\": 0, \"label\": \"模型\"}, {\"name\": \"CONDITIONING+\", \"type\": \"CONDITIONING\", \"links\": [453], \"shape\": 3, \"slot_index\": 1, \"label\": \"正面条件\"}, {\"name\": \"CONDITIONING-\", \"type\": \"CONDITIONING\", \"links\": [454], \"shape\": 3, \"slot_index\": 2, \"label\": \"负面条件\"}, {\"name\": \"LATENT\", \"type\": \"LATENT\", \"links\": [490], \"shape\": 3, \"slot_index\": 3, \"label\": \"Latent\"}, {\"name\": \"VAE\", \"type\": \"VAE\", \"links\": [455], \"shape\": 3, \"slot_index\": 4, \"label\": \"VAE\"}, {\"name\": \"CLIP\", \"type\": \"CLIP\", \"links\": [434], \"shape\": 3, \"slot_index\": 5, \"label\": \"CLIP\"}, {\"name\": \"DEPENDENCIES\", \"type\": \"DEPENDENCIES\", \"links\": \"None\", \"shape\": 3, \"label\": \"依赖\"}], \"properties\": {\"Node name for S&R\": \"Efficient Loader\"}, \"widgets_values\": [\"文-图\\\\二次元\\\\二次元10.safetensors\", \"vae-ft-mse-840000-ema-pruned.safetensors\", -1, \"None\", 1, 1, \"CLIP_POSITIVE\", \"CLIP_NEGATIVE\", \"none\", \"comfy\", 768, 768, 1], \"color\": \"#332222\", \"bgcolor\": \"#553333\", \"shape\": 1}, {\"id\": 23, \"type\": \"LoadImage\", \"pos\": [179, 70], \"size\": {\"0\": 318.8392333984375, \"1\": 357.90679931640625}, \"flags\": {\"pinned\": \"False\"}, \"order\": 6, \"mode\": 0, \"outputs\": [{\"name\": \"IMAGE\", \"type\": \"IMAGE\", \"links\": [492], \"shape\": 3, \"slot_index\": 0, \"label\": \"图像\"}, {\"name\": \"MASK\", \"type\": \"MASK\", \"links\": \"None\", \"shape\": 3, \"label\": \"遮罩\"}], \"properties\": {\"Node name for S&R\": \"LoadImage\"}, \"widgets_values\": [\"下载 (1) (1).png\", \"image\"], \"color\": \"#432\", \"bgcolor\": \"#653\"}], \"links\": [[153, 71, 0, 72, 7, \"SAM_MODEL\"], [207, 72, 0, 69, 0, \"IMAGE\"], [234, 79, 0, 72, 6, \"BBOX_DETECTOR\"], [370, 106, 0, 105, 5, \"UPSCALE_MODEL\"], [418, 105, 0, 120, 0, \"IMAGE\"], [430, 129, 0, 128, 2, \"STRING\"], [431, 130, 0, 128, 3, \"STRING\"], [434, 128, 5, 72, 2, \"CLIP\"], [440, 127, 5, 72, 0, \"IMAGE\"], [443, 113, 0, 128, 1, \"CONTROL_NET_STACK\"], [444, 72, 0, 105, 0, \"IMAGE\"], [446, 79, 1, 72, 8, \"SEGM_DETECTOR\"], [448, 131, 0, 113, 1, \"IMAGE\"], [449, 131, 0, 113, 0, \"IMAGE\"], [450, 131, 0, 113, 2, \"IMAGE\"], [452, 128, 0, 127, 0, \"MODEL\"], [453, 128, 1, 127, 1, \"CONDITIONING\"], [454, 128, 2, 127, 2, \"CONDITIONING\"], [455, 128, 4, 127, 4, \"VAE\"], [458, 127, 0, 72, 1, \"MODEL\"], [459, 127, 1, 72, 4, \"CONDITIONING\"], [460, 127, 2, 72, 5, \"CONDITIONING\"], [461, 127, 4, 72, 3, \"VAE\"], [462, 127, 0, 105, 1, \"MODEL\"], [463, 127, 1, 105, 2, \"CONDITIONING\"], [464, 127, 2, 105, 3, \"CONDITIONING\"], [465, 127, 4, 105, 4, \"VAE\"], [468, 131, 0, 132, 0, \"IMAGE\"], [469, 135, 0, 127, 6, \"INT\"], [490, 128, 3, 127, 3, \"LATENT\"], [492, 23, 0, 131, 1, \"IMAGE\"]], \"groups\": [{\"title\": \"Prompts\", \"bounding\": [135, -72, 910, 617], \"color\": \"#8A8\", \"font_size\": 24, \"locked\": \"False\"}, {\"title\": \"ControlNet\", \"bounding\": [1060, -70, 884, 612], \"color\": \"#8A8\", \"font_size\": 24, \"locked\": \"False\"}, {\"title\": \"High-Res Fix\", \"bounding\": [1819, -1044, 553, 961], \"color\": \"#A88\", \"font_size\": 24, \"locked\": \"False\"}, {\"title\": \"Sampler\", \"bounding\": [132, -1047, 1085, 961], \"color\": \"#3f789e\", \"font_size\": 24, \"locked\": \"False\"}, {\"title\": \"FaceDetailer\", \"bounding\": [1232, -1046, 568, 963], \"color\": \"#8AA\", \"font_size\": 24, \"locked\": \"False\"}], \"config\": {}, \"extra\": {\"ds\": {\"scale\": 0.6209213230591553, \"offset\": {\"0\": 97.73027767011143, \"1\": 755.7798746596844}}}, \"version\": 0.4, \"widget_idx_map\": {\"72\": {\"seed\": 3, \"sampler_name\": 7, \"scheduler\": 8}, \"105\": {\"seed\": 1, \"sampler_name\": 5, \"scheduler\": 6}, \"127\": {\"sampler_name\": 4, \"scheduler\": 5}}, \"seed_widgets\": {\"72\": 3, \"105\": 1}}}}}";
        JSONObject jsonObject = getWorkFlowJSON(workFlowStr, "", picType);

        String color16 = paramObject.getString("color16");
        String width = paramObject.getString("width");
        String height = paramObject.getString("height");

        RestTemplate restTemplate = new RestTemplate();
        HttpEntity<JSONObject> param = new HttpEntity<>(jsonObject);
        ResponseEntity<String> stringResponseEntity = restTemplate.postForEntity("http://121.15.198.153:8188/prompt", param, String.class);

        System.out.println(stringResponseEntity.getBody());
        JSONObject returnJSON = JSONObject.parseObject(stringResponseEntity.getBody());
        if (returnJSON.getInteger("number") == 1) {

        }
        if (returnJSON.getJSONObject("node_errors").size() == 0) {

        }
        String promptId = returnJSON.getString("prompt_id");
        log.info("promptId："+promptId);
//        assert2ComfyUI2save(comfyuiUserWorkflow, jsonObject);
    }

    // 证件照制作
    public void sendWorkFlow5(BaseUserPhotoFolder baseUserPhotoFolder, ComfyuiWorkflow comfyuiWorkflow/*int picType, String picUrl, ComfyuiWorkflowVo comfyuiWorkflowVo, JSONObject paramObject*/) {
//        comfyuiWorkflow.setWorkflowJson(comfyuiUserWorkflow.getWorkflowJson());
//        String workflowJson1 = comfyuiUserWorkflow.getWorkflowJson();
        String workflowJson = comfyuiWorkflow.getWorkflowJson();
        comfyuiWorkflow.setWorkflowJson(workflowJson);
        String appKey = comfyuiWorkflow.getAppKey();
        int picType = comfyuiWorkflow.getPicType();
        if (Strings.isBlank(baseUserPhotoFolder.getPromptJson())) {
            throw new ServerException(ErrorCode.COMFYUI_INPUT_PARAM);
        }
        JSONObject promptObject = JSONObject.parseObject(baseUserPhotoFolder.getPromptJson());

        // 背景颜色
        String color16 = promptObject.getString("color16");
        // 长度
        String widthTmp = promptObject.getString("width");
        // 宽度
        String heightTmp = promptObject.getString("height");
        if (color16.indexOf("#") == -1) {
//            throw new RuntimeException("请输入背景颜色！");
            log.error(baseUserPhotoFolder.getBaseUserId()+" 证件照制作 请输入尺寸！");
            throw new ServerException(ErrorCode.COMFYUI_INPUT_COLOR);
        }
        if (Strings.isBlank(widthTmp) || Strings.isBlank(heightTmp)) {
//            throw new RuntimeException("请输入尺寸！");
            log.error(baseUserPhotoFolder.getBaseUserId()+" 证件照制作 请输入尺寸！");
            throw new ServerException(ErrorCode.COMFYUI_INPUT_SIZE);
        }
        Integer width = Integer.valueOf(widthTmp);
        Integer height = Integer.valueOf(heightTmp);
        // 限制尺寸在512-2048之间
        if(width<512)
            width = 512;
        if(height<512)
            height = 512;
        if(width>2048)
            width = 2048;
        if(height>2048)
            height = 2048;

        int color10 = Integer.parseInt(color16.replace("#", ""), 16);
        JSONObject jsonObject = getWorkFlowJSON(workflowJson, appKey, picType);
        String minioUrl = baseUserPhotoFolder.getUploadPicUrl();//"/comfyui/input/laozhaopianshagnse2.png";
        jsonObject.put("minio_url", minioUrl);
        jsonObject.put("user_workflow_id", picType);
        JSONArray extraDataArray = jsonObject.getJSONObject("extra_data").getJSONObject("extra_pnginfo").getJSONObject("workflow").getJSONArray("nodes").getJSONObject(11).getJSONArray("widgets_values");
        JSONObject prompt = jsonObject.getJSONObject("prompt").getJSONObject("164").getJSONObject("inputs");
        prompt.put("color", color10);
        prompt.put("width", width);
        prompt.put("height", height);
        // 去除模板中的长宽、颜色，再重新添加长宽颜色
        extraDataArray.remove(0);
        extraDataArray.remove(0);
        extraDataArray.remove(0);
        extraDataArray.remove(0);
        extraDataArray.add(width);
        extraDataArray.add(height);
        extraDataArray.add(1);
        extraDataArray.add(color10);
        //
//        sendRestTemplate(jsonObject);
        try {
            assert2ComfyUI2save(baseUserPhotoFolder, jsonObject, null);
        } catch (Exception e) {
            log.error(baseUserPhotoFolder.getBaseUserId()+" 证件照制作异常");
            throw new ServerException(ErrorCode.INTERNAL_SERVER_ERROR);
        }
    }


    // 尺寸外扩
    public void sendWorkFlow8(BaseUserPhotoFolder baseUserPhotoFolder, ComfyuiWorkflow comfyuiWorkflow) {
        if (Strings.isBlank(baseUserPhotoFolder.getPromptJson())) {
            throw new RuntimeException("请输入尺寸！");
        }
        JSONObject promptJson = JSONObject.parseObject( baseUserPhotoFolder.getPromptJson());
        String appKey = comfyuiWorkflow.getAppKey();
        String workFlowStr = comfyuiWorkflow.getWorkflowJson();
        JSONObject jsonObject = getWorkFlowJSON(workFlowStr, appKey, comfyuiWorkflow.getPicType());
        jsonObject.put("minio_url", baseUserPhotoFolder.getUploadPicUrl());
        jsonObject.put("user_workflow_id", comfyuiWorkflow.getPicType());
        int width = promptJson.getInteger("width");
        int height = promptJson.getInteger("height");
        // 计算用户的
        if(width <512) {
            width = 512;
        }
        if (height < 512) {
            height = 512;
        }
        width -= 512;
        height -= 512;
        int top, bottom, left, right;
        top = bottom = height / 2;
        left = right = width / 2;
        JSONObject sizeJSONTmp = jsonObject.getJSONObject("prompt").getJSONObject("12").getJSONObject("inputs");
        sizeJSONTmp.put("top", top);
        sizeJSONTmp.put("bottom", bottom);
        sizeJSONTmp.put("left", left);
        sizeJSONTmp.put("right", right);

        try {
            assert2ComfyUI2save(baseUserPhotoFolder, jsonObject, null);
        } catch (Exception e) {
            log.error(baseUserPhotoFolder.getBaseUserId()+" 证件照制作异常");
            throw new ServerException(ErrorCode.INTERNAL_SERVER_ERROR);
        }
    }


    // 尺寸外扩
    public void sendWorkFlow9(BaseUserPhotoFolder baseUserPhotoFolder, ComfyuiWorkflow comfyuiWorkflow) {
//        if (Strings.isBlank(baseUserPhotoFolder.getPromptJson())) {
//            throw new RuntimeException("请输入尺寸！");
//        }
        String prompt = baseUserPhotoFolder.getPromptJson();
        String appKey = comfyuiWorkflow.getAppKey();
        String workFlowStr = comfyuiWorkflow.getWorkflowJson();
        JSONObject jsonObject = getWorkFlowJSON(workFlowStr, appKey, comfyuiWorkflow.getPicType());
        jsonObject.put("minio_url", baseUserPhotoFolder.getUploadPicUrl());
        jsonObject.put("user_workflow_id", comfyuiWorkflow.getPicType());
//        JSONObject promptObject = JSONObject.parseObject(baseUserPhotoFolder.getPromptJson());
//        int width = promptJson.getString("6");
//        6， {'clip': ['4', 1], 'text': 'beach，sunsine\n'}  14    ['beach，sunsine\n']
//        int height = promptJson.getInteger("height");
//        int top, bottom, left, right;
//        top = bottom = height / 2;
//        left = right = width / 2;
//        JSONObject sizeJSONTmp = jsonObject.getJSONObject("prompt").getJSONObject("6").getJSONObject("inputs");
//        jsonObject.getJSONObject("prompt").getJSONObject("6").put("inputs", prompt);
        jsonObject.getJSONObject("prompt").getJSONObject("6").getJSONObject("inputs").put("text", prompt);
//        jsonObject.getJSONObject("prompt").getJSONObject("6").put("inputs", "prompt");
//        jsonObject.getJSONObject("extra_data").getJSONObject("extra_pnginfo").getJSONObject("workflow").getJSONArray("nodes")
//                .getJSONObject(14).getJSONArray("widgets_values").put("widgets_values", Arrays.asList(prompt));
        JSONArray jsonArray = jsonObject.getJSONObject("extra_data").getJSONObject("extra_pnginfo").getJSONObject("workflow").getJSONArray("nodes").getJSONObject(14).getJSONArray("widgets_values");
        jsonArray.remove(0);
        jsonArray.add(prompt);

        try {
            assert2ComfyUI2save(baseUserPhotoFolder, jsonObject, null);
        } catch (Exception e) {
            log.error(baseUserPhotoFolder.getBaseUserId()+" 证件照制作异常");
            throw new ServerException(ErrorCode.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * 工作流基本的格式的封装
     * @param workFlowStr
     * @param picType
     * @return
     */
    public JSONObject getWorkFlowJSON(String workFlowStr, String appKey, int picType){
        Long userId = SecurityUser.getUserId();
        JSONObject jsonObject = JSONObject.parseObject(workFlowStr);
        // 这边要编写一个规则，查找所有的需要生成图片的数量
//        String appKey = "65df96e17e07a34b576f40a9abe8d8fb";
        StringBuffer randomSB = new StringBuffer();
        for (int i = 0; i < 10; i++) {
            randomSB.append(new Random().nextInt(10));
        }
        String md5 = EncryptUtil.getMD5Byparam(userId + "&" + randomSB + appKey);
        jsonObject.put("random", randomSB.toString());
        jsonObject.put("gen_key", md5);
        jsonObject.put("pic_type", String.valueOf(picType));
        jsonObject.put("client_id", userId);
//        jsonObject.put("callback_gen_pic_url", "http://121.15.198.153:8068/common/api/getComfyUIGenPic2");
        jsonObject.put("callback_gen_pic_url", callbackGenPicUrl);
//        jsonObject.put("callback_gen_pic_url", "http://121.15.198.153:8068/common/api/NumberPersonCommitNotify");
        return jsonObject;
    }

    /**
     * 基于comfyui的SD生成图片
     */
    public void workdFlowSd(BaseUserPhotoFolder baseUserPhotoFolder, ComfyuiWorkflow comfyuiWorkflow/*String workFlowStr, String appKey, int picType*/){

        JSONObject promptJson = JSONObject.parseObject(baseUserPhotoFolder.getPromptJson());
        // 模型名称
        // 文-图\2.8D人物\againmix28D_v1Mature.safetensors
        String modelName = "文-图\\2.8D人物\\againmix28D_v1Mature.safetensors";


        Integer width = promptJson.getInteger("width");
        Integer height = promptJson.getInteger("height");
        if(null==width || width<512 || width>1024){
            throw new ServerException(ErrorCode.COMFYUI_INPUT_WIDTH_SIZE);
        }
        if(null==height || height<512 || height>1024){
            throw new ServerException(ErrorCode.COMFYUI_INPUT_HEIGHT_SIZE);
        }

        // 正向提示词
        String primitive = promptJson.getString("primitive");
        // 反向提示词
        String oppositePrimitive = promptJson.getString("oppositePrimitive");
//        if (Strings.isBlank(primitive)) {
//            throw new ServerException(ErrorCode.COMFYUI_INPUT_PRIMITIVE);
//        }
//        if (Strings.isBlank(oppositePrimitive)) {
//            throw new ServerException(ErrorCode.COMFYUI_INPUT_OPPOSITEPRIMITIVE);
//        }

        if(width <512) {
            width = 512;
        }
        if (height < 512) {
            height = 512;
        }
        if (width>2048)
            width = 2048;
        if(height>2048)
            height = 2048;


        JSONObject jsonObject = getWorkFlowJSON(comfyuiWorkflow.getWorkflowJson(), comfyuiWorkflow.getAppKey(), comfyuiWorkflow.getPicType());
        jsonObject.put("minio_url", baseUserPhotoFolder.getUploadPicUrl());
        jsonObject.put("user_workflow_id", comfyuiWorkflow.getPicType());

        JSONArray extraDataNodes = jsonObject.getJSONObject("extra_data").getJSONObject("extra_pnginfo").getJSONObject("workflow").getJSONArray("nodes");
        JSONObject promptJSON = jsonObject.getJSONObject("prompt");

        // 尺寸宽度、高度
        JSONArray extraHeighWidth = extraDataNodes.getJSONObject(21).getJSONArray("widgets_values");
        extraHeighWidth.remove(1);
        extraHeighWidth.remove(1);
        extraHeighWidth.add(1, width); // 宽度
        extraHeighWidth.add(2, height); // 高度

        JSONArray extraHeighWidth2 = extraDataNodes.getJSONObject(15).getJSONArray("widgets_values");
        extraHeighWidth2.remove(1);
        extraHeighWidth2.remove(1);
        extraHeighWidth2.add(1, width); // 宽度
        extraHeighWidth2.add(2, height);

        promptJSON.getJSONObject("7").getJSONObject("inputs").getString("text");// (hands),text, error,cropped,(worst quality:1.2)，(lowquality:1.2),
        promptJSON.getJSONObject("19").getJSONObject("inputs").put("width", width);
        promptJSON.getJSONObject("19").getJSONObject("inputs").put("height", height);
        promptJSON.getJSONObject("29").getJSONObject("inputs").put("width", width);
        promptJSON.getJSONObject("29").getJSONObject("inputs").put("height", height);

        extraDataNodes.getJSONObject(19).getJSONArray("widgets_values").getJSONObject(0).put("content", modelName);
        jsonObject.getJSONObject("prompt").getJSONObject("11").getJSONObject("inputs").getJSONObject("ckpt_name").put("content", modelName);

        // 正向提示词赋值
        promptJSON.getJSONObject("6").getJSONObject("inputs").put("text", primitive);
        JSONArray jsonArray = extraDataNodes.getJSONObject(0).getJSONArray("widgets_values");//.getString(0)
        jsonArray.remove(0);
        jsonArray.add(primitive);
        JSONArray jsonArray1 = extraDataNodes.getJSONObject(17).getJSONArray("widgets_values");
        jsonArray1.remove(0);
        jsonArray1.add(primitive);
        // 反向提示词赋值
        promptJSON.getJSONObject("7").getJSONObject("inputs").put("text", oppositePrimitive);
        JSONArray oppPrimitJSON = extraDataNodes.getJSONObject(1).getJSONArray("widgets_values");
        oppPrimitJSON.remove(0);
        oppPrimitJSON.add(oppositePrimitive);

        try {
            assert2ComfyUI2save(baseUserPhotoFolder, jsonObject, null);
        } catch (Exception e) {
            log.error(baseUserPhotoFolder.getBaseUserId()+" 证件照制作异常");
            e.printStackTrace();
            throw new ServerException(ErrorCode.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * 基于comfyui的SD的prompt(没有上传图片的文生图)
     */
    public void wordSD_NoPic(BaseUserPhotoFolder baseUserPhotoFolder, ComfyuiWorkflow comfyuiWorkflow/*String workFlowStr, String appKey, int picType*/){
        JSONObject promptJson = JSONObject.parseObject(baseUserPhotoFolder.getPromptJson());
        // 模型名称
        // 文-图\2.8D人物\againmix28D_v1Mature.safetensors
        String modelName = "文-图\\2.8D人物\\againmix28D_v1Mature.safetensors";

        Integer width = promptJson.getInteger("width");
        Integer height = promptJson.getInteger("height");

        // 正向提示词
        String primitive = promptJson.getString("primitive");
        // 反向提示词
        String oppositePrimitive = promptJson.getString("oppositePrimitive");
        if (Strings.isBlank(primitive)) {
            throw new ServerException(ErrorCode.COMFYUI_INPUT_PRIMITIVE);
        }
        if (Strings.isBlank(oppositePrimitive)) {
            throw new ServerException(ErrorCode.COMFYUI_INPUT_OPPOSITEPRIMITIVE);
        }
        if(null==width || width<512 || width>1024){
            throw new ServerException(ErrorCode.COMFYUI_INPUT_WIDTH_SIZE);
        }
        if(null==height || height<512 || height>1024){
            throw new ServerException(ErrorCode.COMFYUI_INPUT_HEIGHT_SIZE);
        }

        JSONObject jsonObject = getWorkFlowJSON(comfyuiWorkflow.getWorkflowJson(), comfyuiWorkflow.getAppKey(), comfyuiWorkflow.getPicType());
        jsonObject.put("user_workflow_id", comfyuiWorkflow.getPicType());

        JSONArray extraDataNodes = jsonObject.getJSONObject("extra_data").getJSONObject("extra_pnginfo").getJSONObject("workflow").getJSONArray("nodes");
        JSONObject promptJSON = jsonObject.getJSONObject("prompt");
        // 尺寸宽度、高度
        JSONArray extraHeigh16Width = extraDataNodes.getJSONObject(16).getJSONArray("widgets_values");
        extraHeigh16Width.remove(1);
        extraHeigh16Width.remove(1);
        extraHeigh16Width.add(1, width); // 宽度
        extraHeigh16Width.add(2, height); // 高度
        JSONArray extraHeigh17Width = extraDataNodes.getJSONObject(17).getJSONArray("widgets_values");
        extraHeigh17Width.remove(1);
        extraHeigh17Width.remove(1);
        extraHeigh17Width.add(1, width); // 宽度
        extraHeigh17Width.add(2, height); // 高度
        promptJSON.getJSONObject("19").getJSONObject("inputs").put("width", width);
        promptJSON.getJSONObject("19").getJSONObject("inputs").put("height", height);
        promptJSON.getJSONObject("29").getJSONObject("inputs").put("width", width);
        promptJSON.getJSONObject("29").getJSONObject("inputs").put("height", height);

//        // 正向提示词
//        String primitive = "An 80-year-old  boys in classroom";
//        // 反向提示词
//        String oppositePrimitive = "no white beard, no white hair, (hands),text, error,cropped,(worst quality:1.2)，(lowquality:1.2),\n" +
//                "normel quality,(jpeg ertifacts:1.3),signature,watermark, \n" +
//                "username,blurry,artist name,monochrome,sketch,censorship, \n" +
//                "censor,(copyright:1.2),extra legs,(foreheadmark)(depth of field)(emotionless)";
        // 正向提示词赋值
        promptJSON.getJSONObject("6").getJSONObject("inputs").put("text", primitive);
        JSONArray jsonArray = extraDataNodes.getJSONObject(0).getJSONArray("widgets_values");//.getString(0)
        jsonArray.remove(0);
        jsonArray.add(primitive);

        JSONArray primitArray = extraDataNodes.getJSONObject(15).getJSONArray("widgets_values");
        primitArray.remove(0);
        primitArray.add(primitive);

        // 反向提示词赋值
        promptJSON.getJSONObject("7").getJSONObject("inputs").put("text", oppositePrimitive);
        JSONArray oppPrimitJSON = extraDataNodes.getJSONObject(1).getJSONArray("widgets_values");
        oppPrimitJSON.remove(0);
        oppPrimitJSON.add(oppositePrimitive);

        JSONArray opp2Array = extraDataNodes.getJSONObject(11).getJSONArray("widgets_values");
        opp2Array.remove(0);
        opp2Array.add(oppositePrimitive);

        try {
            assert2ComfyUI2save(baseUserPhotoFolder, jsonObject, null);
        } catch (Exception e) {
            log.error(baseUserPhotoFolder.getBaseUserId()+" 证件照制作异常");
            throw new ServerException(ErrorCode.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * 发送请求
     * @param jsonObject
     */
    public String sendRestTemplate(JSONObject jsonObject){
        String promptId = "";
        HttpHeaders headers = new HttpHeaders();
        headers.add("Content-Type","application/json;charset=UTF-8");
        RestTemplate restTemplate = new RestTemplate();
        HttpEntity<JSONObject> param = new HttpEntity<>(jsonObject, headers);
        try {
            ResponseEntity<String> stringResponseEntity = restTemplate.postForEntity(comfyuiUrl, param, String.class);

            log.info(jsonObject.getString("client_id") + "推送comfyui报文：" + stringResponseEntity.getBody());
            JSONObject returnJSON = JSONObject.parseObject(stringResponseEntity.getBody());
            promptId = returnJSON.getString("prompt_id");
            log.info(jsonObject.getString("client_id")+"推送comfyui，返回promptId：" + promptId);
        }catch (Exception e){
            log.error("发送请求失败",e.getMessage());
            // 短信发送失败

        }
        return promptId;
    }
}

