package com.xyzc.start.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.xyzc.common.core.exception.ServerException;
import com.xyzc.common.web.security.user.SecurityUser;
import com.xyzc.start.constant.FeedbackConst;
import com.xyzc.start.domain.HelpContent;
import com.xyzc.start.domain.BaseUserFeedback;
import com.xyzc.start.domain.bo.HelpFeedbackBo;
import com.xyzc.start.mapper.HelpCommonMapper;
import com.xyzc.start.mapper.BaseUserFeedbackMapper;
import com.xyzc.start.service.IHelpPageService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@RequiredArgsConstructor
public class HelpPageServiceImpl implements IHelpPageService {

    private final BaseUserFeedbackMapper helpFeedbackmapper;
    private final HelpCommonMapper helpCommonMapper;

    /**
     * 提交反馈
     *
     * @param helpFeedbackBo
     */
    @Override
    public void submit(HelpFeedbackBo helpFeedbackBo) {
        BaseUserFeedback baseUserFeedback = new BaseUserFeedback();
        baseUserFeedback.setBaseUserId(SecurityUser.getUserId());
        baseUserFeedback.setContent(helpFeedbackBo.getContent());
        baseUserFeedback.setFeedbackType(helpFeedbackBo.getFeedbackType());

        boolean flag = helpFeedbackmapper.insert(baseUserFeedback) == 1;
        if (!flag) {
            throw new RuntimeException("反馈提交失败");
        }
    }


    /**
     * 对帮助文档打分
     *
     * @param id
     * @param status
     */
    @Override
    public void setHelper(Long id, Integer status) {
        HelpContent helpCommon = helpCommonMapper.selectById(id);
        if (helpCommon == null) {
            throw new ServerException("反馈失败，请联系商家");
        }
        if (FeedbackConst.HELPER.equals(status)) {
            helpCommonMapper.update(new LambdaUpdateWrapper<HelpContent>()
                    .set(HelpContent::getHelpful, helpCommon.getHelpful() + 1)
                    .eq(HelpContent::getId, id));
        } else {
            helpCommonMapper.update(new LambdaUpdateWrapper<HelpContent>()
                    .set(HelpContent::getNoHelp, helpCommon.getNoHelp() + 1)
                    .eq(HelpContent::getId, id));
        }
    }

    /**
     * 根据id 查询能够返回的帮助文档
     *
     * @param id
     * @return
     */
    @Override
    public List<HelpContent> queryHelpDocById(Long id) {
        //每次都只拿6个
        List<HelpContent> helpDocs = helpCommonMapper.queryHelpDocListById(id != null ? id : 0);

        return helpDocs;
    }
}
