package com.xyzc.start.service.impl;

import com.xyzc.common.core.exception.ErrorCode;
import com.xyzc.common.core.exception.ServerException;
import com.xyzc.common.core.utils.AssertUtils;
import com.xyzc.common.core.utils.MapstructUtils;
import com.xyzc.common.core.utils.SpringUtils;
import com.xyzc.common.web.security.user.SecurityUser;
import com.xyzc.start.constant.BaseUserOrderStatus;
import com.xyzc.start.constant.SchemeConst;
import com.xyzc.start.domain.BaseUserOrder;
import com.xyzc.start.domain.BaseUserOrderDetail;
import com.xyzc.start.domain.Scheme;
import com.xyzc.start.domain.bo.CreateOrderBo;
import com.xyzc.start.domain.vo.*;
import com.xyzc.start.mapper.*;
import com.xyzc.start.service.ISchemeService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Service
@RequiredArgsConstructor
public class SchemeServiceImpl implements ISchemeService {

    private final SchemeMapper schemeMapper;
    private final MemberCardMapper memberCardMapper;
    private final EquityMapper equityMapper;
    private final AiModelMapper aiModelMapper;

    private final FuelPackageMapper fuelPackageMapper;
    private final BaseUserOrderMapper baseUserOrderMapper;
    private final BaseUserOrderDetailMapper baseUserOrderDetailMapper;

    /**
     * 获取当前方案
     *
     * @return
     */
    @Override
    public SchemeVo getCurrentMember() {
        SchemeVo scheme = schemeMapper.getCurrentMember();

        // 有且只有一个会员卡套餐生效
        // 根据套餐查询会员卡
        // TODO 代理商id暂时写死
        List<MemberCardVo> memberCardVos = memberCardMapper.queryVoListBySchemeId(scheme.getId(), 1810310843224322049L);
        // 让memberCardVos中根据memberId合并成新的List
        List<List<MemberCardVo>> memberList = new ArrayList<>();
        List<MemberCardVo> collect = new ArrayList<>();
        for (MemberCardVo memberCardVo : memberCardVos) {
            // collect不为空并且memberCardId不一致时新建一个list
            if (!collect.isEmpty() && !collect.get(0).getMemberCardId().equals(memberCardVo.getMemberCardId())) {
                memberList.add(collect);
                collect = new ArrayList<>();
            }
            // 对当前值进行处理
            memberCardVo.setSaveMoney(memberCardVo.getPrice().subtract(memberCardVo.getStrikethroughPrice()));
            collect.add(memberCardVo);
            memberCardVo.setMemberEquityList(grouth());

        }
        memberList.add(collect);

        // 将会员卡列表绑定到方案中
        scheme.setMemberCardList(memberList);

        return scheme;
    }

    private static List<MemberEquityVo> grouth() {
        ArrayList<MemberEquityVo> objects = new ArrayList<>();

        MemberEquityVo memberCardVo = new MemberEquityVo();
        memberCardVo.setMemberEquityid(1);
        memberCardVo.setMemberEquityName("应用权益");
        memberCardVo.setMemberEquityCount(230);

        MemberEquityVo memberCardVo2 = new MemberEquityVo();
        memberCardVo2.setMemberEquityid(2);
        memberCardVo2.setMemberEquityName("AI绘画");
        memberCardVo2.setMemberEquityCount(2320);

        MemberEquityVo memberCardVo3 = new MemberEquityVo();
        memberCardVo3.setMemberEquityid(2);
        memberCardVo3.setMemberEquityName("AI绘画");
        memberCardVo3.setMemberEquityCount(2320);

        MemberEquityVo memberCardVo4 = new MemberEquityVo();
        memberCardVo4.setMemberEquityid(2);
        memberCardVo4.setMemberEquityName("AI绘画");
        memberCardVo4.setMemberEquityCount(2320);

        MemberEquityVo memberCardVo5 = new MemberEquityVo();
        memberCardVo5.setMemberEquityid(2);
        memberCardVo5.setMemberEquityName("AI绘画");
        memberCardVo5.setMemberEquityCount(2320);

        return objects;
    }


    /**
     * 分页查询燃料规则详情
     *
     * @param modelId 模型id
     * @return
     */
    @Override
    public List<AiModelLawVo> queryModelLawList(Long modelId) {

        return aiModelMapper.queryModelLawList(modelId);
    }

    /**
     * 根据会员卡id获取会员卡信息
     *
     * @param memberId
     * @return
     */
    @Override
    public MemberCardVo queryCurrentMemberById(Long memberId) {

        // 传入用户自己的值去
        return schemeMapper.queryMemberCardByMemberId(SecurityUser.getUserId(), memberId);
    }

    /**
     * 获取当前生效的燃料方案
     *
     * @return
     */
    @Override
    public SchemeVo getCurrentFuel() {
        List<SchemeVo> currentFuel = schemeMapper.getCurrentFuel();
        // 有且只有一个会员卡套餐生效
        AssertUtils.isUnique(currentFuel, ErrorCode.SCHEME_NOT_UNIQUE);
        SchemeVo scheme = currentFuel.get(0);

        // 根据套餐查询燃料
        List<FuelPackageVo> fuelPackageVo = schemeMapper.queryFuelPackageById(scheme.getId());

        scheme.setFuelPackageList(fuelPackageVo);

        return scheme;
    }

    /**
     * 创建订单
     *
     * @param bo
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseUserOrderVo createOrder(CreateOrderBo bo) {
        // 生成订单
        BaseUserOrderVo baseUserOrderVo = new BaseUserOrderVo();
        baseUserOrderVo.setBaseUserId(SecurityUser.getUserId());

        List<MemberCardOrderVo> memberCardOrderList = null;
        // 或者燃料包
        List<FuelPackageOrderVo> fuelPackageOrderList = null;

        // 判断套餐是否生效
        if (SchemeConst.MEMEBER_TYPE.equals(bo.getType())) {
            Scheme scheme = schemeMapper.querySchemeByMemberId(bo);
            // 没有生效的套餐抛出异常
            AssertUtils.notNull(scheme, ErrorCode.SCHEME_NOT_EXIST);
            memberCardOrderList = memberCardMapper.queryOrderByMemberId(bo, 1810310843224322049L);

            baseUserOrderVo.setGoodsAmount(memberCardOrderList.get(0).getPrice());
            baseUserOrderVo.setPayAmount(memberCardOrderList.get(0).getPrice());

        } else if (SchemeConst.FUEL_TYPE.equals(bo.getType())) {
            Scheme scheme = schemeMapper.querySchemeByFuel(bo);
            // 没有生效的套餐抛出异常
            AssertUtils.notNull(scheme, ErrorCode.SCHEME_NOT_EXIST);
            fuelPackageOrderList = fuelPackageMapper.queryOrderByFuelPackageId(bo, 1810310843224322049L);

            baseUserOrderVo.setGoodsAmount(fuelPackageOrderList.get(0).getPrice());
            baseUserOrderVo.setPayAmount(fuelPackageOrderList.get(0).getPrice());
        }

        // 设置订单信息
        baseUserOrderVo.setMemberCardList(memberCardOrderList);
        baseUserOrderVo.setFuelPackageList(fuelPackageOrderList);
        baseUserOrderVo.setStatus(BaseUserOrderStatus.UNPAID);
        baseUserOrderVo.setDiscountAmount(new BigDecimal(0));

        // 插入订单，返回订单id
        BaseUserOrder convert = MapstructUtils.convert(baseUserOrderVo, BaseUserOrder.class);
        AssertUtils.notNull(convert, ErrorCode.ORDER_NO_EXIST);
        convert.setBaseUserId(SecurityUser.getUserId());
        // todo 后期写活
        convert.setAgentMerId(1810310843224322049L);
        convert.setBaseUserId(SecurityUser.getUserId());
        baseUserOrderMapper.insert(convert);
        baseUserOrderVo.setId(convert.getId());
        baseUserOrderVo.setCreateTime(convert.getCreateTime());

        // 生成订单详情
        BaseUserOrderDetail baseUserOrderDetail = new BaseUserOrderDetail();
        baseUserOrderDetail.setAgentId(1810310843224322049L);
        baseUserOrderDetail.setGoodsType(bo.getType());
        baseUserOrderDetail.setGoodsId(bo.getId());
        baseUserOrderDetail.setBaseUserOrderId(convert.getId());
        // TODO: 这里因为是单个商品购买，所以只需要输入1即可，后续别的地方需要活用
        baseUserOrderDetail.setGoodsCount(1);
        baseUserOrderDetail.setBaseUserId(SecurityUser.getUserId());

        // 插入会员卡信息或者插入燃料包信息
        if (SchemeConst.MEMEBER_TYPE.equals(bo.getType())) {
            baseUserOrderDetailMapper.insert(baseUserOrderDetail);
        } else if (SchemeConst.FUEL_TYPE.equals(bo.getType())) {
            baseUserOrderDetailMapper.insert(baseUserOrderDetail);
        }

        return baseUserOrderVo;
    }




    /**
     * 消费mq消息，拿到订单支付状态，修改订单状态
     */
//    @Override
//    public void updateOrderStatus(BaseUserOrder baseUserOrder) {
//        if (BaseUserOrderStatus.PAID.equals(baseUserOrder.getStatus())) {
//            baseUserOrderMapper.updateOrderStatus(baseUserOrder);
//        } else {
//            throw new ServerException("订单状态异常");
//        }
//    }


}
